/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;



/**
 * Implementation of the framework used to prepare and run printing tests.
 * Output of such tests are stored in a PostScript file which could be rendered
 * using GhostScript or other implementation of PostScript language.
 * 
 * @author Pavel Tisnovsky
 */
public class PrintingTest implements Printable
{
    /**
     * Used to run the selected test
     */
    private GfxTest gfxTest;

    /**
     * Method of selected test which should be called
     */
    private Method method;

    /**
     * Test result
     */
    private TestResult result = null;

    /**
     * Constructor which configures printing test framework.
     * 
     * @param gfxTest
     *            used to run the selected test
     * @param method
     *            method of selected test which should be called
     */
    public PrintingTest(GfxTest gfxTest, Method method)
    {
        setGfxTest(gfxTest);
        setMethod(method);
    }

    /**
     * Get the current value of attribute method.
     *
     * @return the method
     */
    public Method getMethod()
    {
        return this.method;
    }

    /**
     * Set a new value of a attribute method.
     *
     * @param method the method to set
     */
    public void setMethod(Method method)
    {
        this.method = method;
    }

    /**
     * Get the current value of attribute result.
     *
     * @return the result
     */
    public TestResult getResult()
    {
        return this.result;
    }

    /**
     * Set a new value of a attribute method.
     *
     * @param result the result to set
     */
    public void setResult(TestResult result)
    {
        this.result = result;
    }

    /**
     * Get the current value of attribute gfxTest.
     *
     * @return the gfxTest
     */
    public GfxTest getGfxTest()
    {
        return this.gfxTest;
    }

    /**
     * Set a new value of a attribute gfxTest.
     *
     * @param gfxTest the gfxTest to set
     */
    public void setGfxTest(GfxTest gfxTest)
    {
        this.gfxTest = gfxTest;
    }

    /**
     * Prints the page at the specified index into the specified Graphics context in the specified format.
     */
    @Override
    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException
    {
        this.gfxTest.log.log(this.getClass().getName(), "printBegin " + pageIndex);
        /* we have only one page, and 'page' is zero-based */
        if (pageIndex > 0) { 
            this.gfxTest.log.log(this.getClass().getName(), "printEnd no such page " + pageIndex);
            return NO_SUCH_PAGE;
        }

        // this should not usually happen, but make sure we
        // got a proper instance of Graphics2D
        if (!(graphics instanceof Graphics2D))
        {
            return 0;
        }

        Graphics2D gc = (Graphics2D)graphics;
        translateCoordinateSystem(pageFormat, gc);

        /* create test image with the width and height based of page format */
        TestImage image = createTestImage(pageFormat);

        try
        {
            setResult((TestResult) this.method.invoke(getGfxTest(), new Object[] { image, gc }));
        }
        catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            setResult(TestResult.ERROR);
        }
        catch (IllegalAccessException e)
        {
            e.printStackTrace();
            setResult(TestResult.ERROR);
        }
        catch (InvocationTargetException e)
        {
            e.printStackTrace();
            setResult(TestResult.ERROR);
        }
        graphics.dispose();
        this.gfxTest.log.log(this.getClass().getName(), "printEnd");

        /* tell the caller that this page is part of the printed document */
        return PAGE_EXISTS;
    }

    /**
     * Translate coordinate system used to put graphics object into a page.
     * 
     * Origin (0,0) is typically outside the imageable area, so we must
     * translate by the X and Y values in the PageFormat to avoid clipping
     * 
     * @param pageFormat
     *            object containing the size and orientation of a page to be
     *            printed.
     * @param gc graphics context
     */
    private void translateCoordinateSystem(PageFormat pageFormat, Graphics2D gc)
    {
        gc.translate(pageFormat.getImageableX() * 2, pageFormat.getImageableY() * 2);
    }

    /**
     * Create test image with the width and height based of page format.
     * 
     * @param pageFormat
     *            object containing the size and orientation of a page to be
     *            printed.
     * @return object representing raster image on which shapes are rendered.
     */
    private TestImage createTestImage(PageFormat pageFormat)
    {
        final double width = pageFormat.getImageableWidth();
        final double height = pageFormat.getImageableHeight();
        final double xoffset = pageFormat.getImageableX();
        final double yoffset = pageFormat.getImageableY();
        return new TestImage(width - xoffset, height - yoffset);
    }

}
